package org.greenstone.gsdl3;

import org.greenstone.gsdl3.core.*;
import org.greenstone.gsdl3.util.*;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.Enumeration;
import java.io.File;

/** a servlet to serve the greenstone library - we are using servlets instead
 * of cgi
 * the init method is called only once - the first time the servlet classes
 * are loaded. Each time a request comes in to the servlet, the session()
 * method is called in a new thread (calls doGet/doPut etc)
 * takes the a=p&p=home type args and builds a simple request to send to 
 * its receptionist, which returns a result in html, cos output=html
 * is set in the request
 * @see Receptionist
 */
public class LibraryServlet extends HttpServlet {
    
    /** the receptionist to send messages to */
    protected Receptionist recept_=null;
    /** the default language - is specified by setting a servlet param,
     * otherwise DEFUALT_LANG is used*/
    protected String default_lang_= null;
    /** The default default - used if a default lang is not specified 
     * in the servlet params */
    protected final String DEFAULT_LANG = "en";
    /** initialise the servlet
     */
    public void init(ServletConfig config) throws ServletException {
	// always call super.init;
	super.init(config);

	ConfigVars config_vars = new ConfigVars();
	String library_name = config.getInitParameter("libraryname");
	String gsdl3_home = config.getInitParameter("gsdl3home");
	String site_name = config.getInitParameter("sitename");
	String interface_name = config.getInitParameter("interfacename");
	default_lang_ = config.getInitParameter("defaultlang");
	if (library_name == null || gsdl3_home == null || site_name ==null ||  interface_name ==null) {
	    // must have this
	    System.err.println("initialisation parameters not all set!");
	    System.err.println(" you must have libraryname, gsdl3home, sitename,  interfacename");
	    System.exit(1);
	}

	if (default_lang_ == null) {
	    // choose english
	    default_lang_ = DEFAULT_LANG;
	}

	config_vars.library_name_ = library_name;
	config_vars.gsdl3_home_ = gsdl3_home;
	config_vars.site_name_ = site_name;
	config_vars.interface_name_ =interface_name;
	config_vars.default_language_ = default_lang_;
	config_vars.createXML();
	
	// new message router - create it and pass reference to recept.
	// the servlet wont use this directly
	MessageRouter message_router = new MessageRouter();
	message_router.setSiteHome(GSFile.siteHome(config_vars.gsdl3_home_, 
						    config_vars.site_name_));
	message_router.configure();

	// the receptionist -the servlet will talk to this
	recept_ = new Receptionist();
	recept_.setConfigVars(config_vars);
	recept_.setMessageRouter(message_router);
	recept_.configure();

    }
    
    public void doGet(HttpServletRequest request,
		      HttpServletResponse response)
	throws ServletException, IOException {
	
	request.setCharacterEncoding("UTF-8");
	response.setContentType("text/html;charset=UTF-8");
	PrintWriter out = response.getWriter();

	// if the interface owns the default - keep this here, otherwise move
	// it to receptionist
	String lang = request.getParameter("l");
	if (lang==null || lang.equals("")) {
	    lang = default_lang_; // the default
	}
	// the request to the receptionist	
	String xml_request="<message><request lang='"+lang+"' type='action' output='html' action=";

	String action = request.getParameter("a");
	if (action==null || action.equals("")) {
	    // display the home page
	    // a=p&sa=home
	    xml_request +="'p' subaction='home'/>";
	} else {
	    String subaction = request.getParameter("sa");
	    xml_request += "'"+action+"'";
	    if (subaction != null) {
		xml_request += " subaction='"+subaction+"'";
	    }
	    xml_request += "><paramList>";
	    Enumeration params = request.getParameterNames();
	    while(params.hasMoreElements()) {
		String name = (String)params.nextElement();
		if (!name.equals("a") && !name.equals("sa") && !name.equals("l")) { // these ones are in the header so leave out
		    String value="";
		    String []values = request.getParameterValues(name);
		    value = values[0];
		    if (values.length > 1) {
			for (int i=1; i< values.length; i++) {
			    value += ","+values[i];
			}
		    }
		    xml_request += "<param name='"+name+"' value='"+GSXML.xmlSafe(value)+"'/>";
		    
		}
	    }
	    xml_request += "</paramList></request>";
	    if (action.equals("a")&&subaction.equals("r")) { // applet action
		response.setContentType("text/plain");
	    }
	}
	xml_request += "</message>";

	String result = recept_.process(xml_request);

	out.println(result);


	
    }
}
